# coding: utf-8
from io import BytesIO
from maxipago.utils import etree
from maxipago.resources.base import Resource
from maxipago.exceptions import PaymentException


class TransactionResource(Resource):

    def process(self):    
        tree = etree.parse(BytesIO(self.data))
        
        header = tree.find('header')
        error_code = header.find('errorCode')
        
        if error_code is not None and error_code.text != '0':
            error_message = '' #header.find('errorMsg').text
            print(etree.tostring(tree, pretty_print=False))
            raise PaymentException(message=error_message)
        
        result = tree.find('result')
        conf = result.find('resultSetInfo')
        
        self.pages = conf.find('totalNumberOfRecords').text if conf.find('totalNumberOfRecords') is not None else 0
        self.page_number = conf.find('pageNumber').text if conf.find('pageNumber') is not None else None
        self.page_token = conf.find('pageToken').text if conf.find('pageToken') is not None else None
        self.items = []
        records = result.find('records')
        for element in records.iter('record'):
            #print(etree.tostring(element, pretty_print=False))
            #print('-----')
            fields = [
                ('transactionId', 'transaction_id'),
                ('referenceNummber', 'reference_num'),
                ('transactionType', 'transaction_type'),
                ('transactionAmount', 'transaction_amount'),
                ('shippingAmount', 'shipping_amount'),
                ('transactionDate', 'transaction_date'),
                ('orderId', 'order_id'),
                ('splitPaymentOrderId', 'split_payment_order_id'),
                ('userId', 'user_id'),
                ('customerId', 'customer_id'),
                ('companyName', 'company_name'),                
                ('responseCode', 'response_code'),
                ('approvalCode', 'approval_code'),
                ('paymentType', 'payment_type'),
                ('bankRoutingNumber', 'bank_routing_number'),
                ('achAccountNumber', 'ach_account_number'),
                ('avsResponseCode', 'avs_response_code'),
                ('billingAddress1', 'billing_address1'),
                ('billingName', 'billing_name'),
                ('billingAddress2', 'billing_address2'),
                ('billingCity', 'billing_city'),
                ('billingState', 'billing_state'),
                ('billingCountry', 'billing_country'),
                ('billingZip', 'billing_zip'),
                ('billingPhone', 'billing_phone'),
                ('billingEmail', 'billing_email'),
                ('comments', 'comments'),
                ('transactionStatus', 'transaction_status'),
                ('transactionState', 'transaction_state'),
                ('recurringPaymentFlag', 'recurring_payment_flag'),
                ('processorReturnedData', 'processor_returned_data'),
                ('gatewayDebitNetworkID', 'gateway_debit_network_id'),
                ('creditCardType', 'credit_card_type'),
                ('boletoUrl', 'boleto_url'),
                ('boletoNumber', 'boleto_number'),
                ('expirationDate', 'expiration_date'),
                ('processorID', 'processor_id'),
                ('dateOfPayment', 'date_of_payment'),
                ('dateOfFunding', 'date_of_funding'),
                ('bankOfPayment', 'bank_of_payment'),
                ('branchOfPayment', 'branch_of_payment'),
                ('paidAmount', 'paid_amount'),
                ('bankFee', 'bank_fee'),
                ('netAmount', 'net_amount'),
                ('returnCode', 'return_code'),
                ('clearingCode', 'clearing_code'),
                ('customField1', 'custom_field1'),
                ('customField2', 'custom_field2'),
                ('customField3', 'custom_field3'),
                ('customField4', 'custom_field4'),
                ('customField5', 'custom_field5'),
                ('numberOfInstallments', 'number_of_installments'),
                ('chargeInterest', 'charge_interest'),
                ('processorTransactionID', 'processor_transaction_id'),
                ('processorReferenceNumber', 'processor_reference_number')
            ]
            item = {}
            for f_name, f_translated in fields:
                field = element.find(f_name)
                if field is not None:                    
                    item.update({f_translated: field.text})
            if item is not None:
                self.items.append(item)


# coding: utf-8
from io import StringIO
from maxipago.utils import etree
from maxipago.resources.base import Resource
from maxipago.exceptions import CustomerAlreadyExists, CustomerException


class CustomerAddResource(Resource):

    def process(self):
        tree = etree.parse(StringIO(self.data))

        error_code = tree.find('errorCode').text

        if error_code != '0':
            error_message = tree.find('errorMessage').text

            if 'already exists' in error_message.lower():
                raise CustomerAlreadyExists(message=error_message)

            raise CustomerException(message=error_message)

        self._meta = {
            'command': tree.find('command').text,
            'time': tree.find('time').text,
        }

        self.id = tree.find('result').find('customerId').text


class CustomerDeleteResource(Resource):

    def process(self):
        tree = etree.parse(StringIO(self.data))

        error_code = tree.find('errorCode').text

        if error_code != '0':
            error_message = tree.find('errorMessage').text

            raise CustomerException(message=error_message)

        self._meta = {
            'command': tree.find('command').text,
            'time': tree.find('time').text,
        }

        self.success = True


class CustomerUpdateResource(Resource):

    def process(self):
        tree = etree.parse(StringIO(self.data))

        error_code = tree.find('errorCode').text

        if error_code != '0':
            error_message = tree.find('errorMessage').text

            raise CustomerException(message=error_message)

        self._meta = {
            'command': tree.find('command').text,
            'time': tree.find('time').text,
        }

        self.success = True



# coding: utf-8


class MaxipagoException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class ValidationError(MaxipagoException):
    def __repr__(self):
        return 'ValidationError(%s)' % self.message


# customer
class CustomerException(MaxipagoException):
    def __repr__(self):
        return 'CustomerError(%s)' % self.message


class CustomerAlreadyExists(CustomerException):
    def __repr__(self):
        return 'CustomerValidation(%s)' % self.message


class CardException(MaxipagoException):
    def __repr__(self):
        return 'CardError(%s)' % self.message


class PaymentException(MaxipagoException):
    def __repr__(self):
        return 'PaymentError(%s)' % self.message


#http
class HttpErrorException(MaxipagoException):
    def __repr__(self):
        return 'HttpError(%s)' % self.message



# coding: utf-8


class Resource(object):

    def __init__(self, data, requester, manager):
        self.data = data
        self.requester = requester
        self.manager = manager
        self.process()

    def process(self):
        raise NotImplementedError()



# coding: utf-8
from urllib.parse import urlencode
from hashlib import md5
from maxipago.managers.base import ManagerTransaction
from maxipago.requesters.payment import PaymentRequester
from maxipago.resources.payment import PaymentResource


class PaymentManager(ManagerTransaction):

    def authorize(self, **kwargs):
        fields = (
            ('processor_id', {'translated_name': 'processorID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),
            ('fraud_check', {'translated_name': 'fraudCheck', 'required': False}),
            ('ip_address', {'translated_name': 'ipAddress', 'required': False}),

            ('billing_name', {'translated_name': 'billing/name', 'required': False}),
            ('billing_address', {'translated_name': 'billing/address', 'required': False}),
            ('billing_address2', {'translated_name': 'billing/address2', 'required': False}),
            ('billing_city', {'translated_name': 'billing/city', 'required': False}),
            ('billing_state', {'translated_name': 'billing/state', 'required': False}),
            ('billing_postalcode', {'translated_name': 'billing/postalcode', 'required': False}),
            ('billing_country', {'translated_name': 'billing/country', 'required': False}),
            ('billing_phone', {'translated_name': 'billing/phone', 'required': False}),
            ('billing_email', {'translated_name': 'billing/email', 'required': False}),

            ('shipping_name', {'translated_name': 'shipping/name', 'required': False}),
            ('shipping_address', {'translated_name': 'shipping/address', 'required': False}),
            ('shipping_address2', {'translated_name': 'shipping/address2', 'required': False}),
            ('shipping_city', {'translated_name': 'shipping/city', 'required': False}),
            ('shipping_state', {'translated_name': 'shipping/state', 'required': False}),
            ('shipping_postalcode', {'translated_name': 'shipping/postalcode', 'required': False}),
            ('shipping_country', {'translated_name': 'shipping/country', 'required': False}),
            ('shipping_phone', {'translated_name': 'shipping/phone', 'required': False}),
            ('shipping_email', {'translated_name': 'shipping/email', 'required': False}),

            ('card_number', {'translated_name': 'transactionDetail/payType/creditCard/number', 'required': False}),
            ('card_expiration_month', {'translated_name': 'transactionDetail/payType/creditCard/expMonth', 'required': False}),
            ('card_expiration_year', {'translated_name': 'transactionDetail/payType/creditCard/expYear', 'required': False}),
            ('card_cvv', {'translated_name': 'transactionDetail/payType/creditCard/cvvNumber', 'required': False}),

            ('customer_id', {'translated_name': 'transactionDetail/payType/onFile/customerId', 'required': False}),
            ('token', {'translated_name': 'transactionDetail/payType/onFile/token', 'required': False}),

            ('charge_total', {'translated_name': 'payment/chargeTotal'}),
            ('installments', {'translated_name': 'payment/creditInstallment/numberOfInstallments', 'required': False}),
            ('installments_interest', {'translated_name': 'payment/creditInstallment/chargeInterest', 'required': False}),
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='auth', requester=requester, resource=PaymentResource)

    def capture(self, **kwargs):
        fields = (
            ('order_id', {'translated_name': 'orderID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),

            ('ip_address', {'translated_name': 'ipAddress', 'required': False}),
            ('charge_total', {'translated_name': 'payment/chargeTotal'}),
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='capture', requester=requester, resource=PaymentResource)

    def direct(self, **kwargs):
        fields = (
            ('processor_id', {'translated_name': 'processorID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),
            ('fraud_check', {'translated_name': 'fraudCheck', 'required': False}),

            ('ip_address', {'translated_name': 'ipAddress', 'required': False}),

            ('billing_name', {'translated_name': 'billing/name', 'required': False}),
            ('billing_address', {'translated_name': 'billing/address', 'required': False}),
            ('billing_address2', {'translated_name': 'billing/address2', 'required': False}),
            ('billing_city', {'translated_name': 'billing/city', 'required': False}),
            ('billing_state', {'translated_name': 'billing/state', 'required': False}),
            ('billing_postalcode', {'translated_name': 'billing/postalcode', 'required': False}),
            ('billing_country', {'translated_name': 'billing/country', 'required': False}),
            ('billing_phone', {'translated_name': 'billing/phone', 'required': False}),
            ('billing_email', {'translated_name': 'billing/email', 'required': False}),

            ('shipping_name', {'translated_name': 'shipping/name', 'required': False}),
            ('shipping_address', {'translated_name': 'shipping/address', 'required': False}),
            ('shipping_address2', {'translated_name': 'shipping/address2', 'required': False}),
            ('shipping_city', {'translated_name': 'shipping/city', 'required': False}),
            ('shipping_state', {'translated_name': 'shipping/state', 'required': False}),
            ('shipping_postalcode', {'translated_name': 'shipping/postalcode', 'required': False}),
            ('shipping_country', {'translated_name': 'shipping/country', 'required': False}),
            ('shipping_phone', {'translated_name': 'shipping/phone', 'required': False}),
            ('shipping_email', {'translated_name': 'shipping/email', 'required': False}),

            ('card_number', {'translated_name': 'transactionDetail/payType/creditCard/number', 'required': False}),
            ('card_expiration_month', {'translated_name': 'transactionDetail/payType/creditCard/expMonth', 'required': False}),
            ('card_expiration_year', {'translated_name': 'transactionDetail/payType/creditCard/expYear', 'required': False}),
            ('card_cvv', {'translated_name': 'transactionDetail/payType/creditCard/cvvNumber', 'required': False}),

            ('customer_id', {'translated_name': 'transactionDetail/payType/onFile/customerId', 'required': False}),
            ('token', {'translated_name': 'transactionDetail/payType/onFile/token', 'required': False}),

            ('charge_total', {'translated_name': 'payment/chargeTotal'}),
            ('installments', {'translated_name': 'payment/creditInstallment/numberOfInstallments', 'required': False}),
            ('installments_interest', {'translated_name': 'payment/creditInstallment/chargeInterest', 'required': False}),
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='sale', requester=requester, resource=PaymentResource)

    def cancel(self, **kwargs):
        fields = (
            ('transaction_id', {'translated_name': 'transactionID'}),
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='void', requester=requester, resource=PaymentResource)

    def chargeback(self, **kwargs):
        fields = (
            ('order_id', {'translated_name': 'orderID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),
            ('charge_total', {'translated_name': 'payment/chargeTotal'}),
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='return', requester=requester, resource=PaymentResource)

    def bank_slip(self, **kwargs):
        fields = (
            ('processor_id', {'translated_name': 'processorID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),
            ('customerid_ext', {'translated_name': 'customerIdExt'}),
            
            ('ip_address', {'translated_name': 'ipAddress', 'required': False}),

            ('billing_name', {'translated_name': 'billing/name'}),
            ('billing_address', {'translated_name': 'billing/address', 'required': False}),
            ('billing_address2', {'translated_name': 'billing/address2', 'required': False}),
            ('billing_city', {'translated_name': 'billing/city', 'required': False}),
            ('billing_state', {'translated_name': 'billing/state', 'required': False}),
            ('billing_postalcode', {'translated_name': 'billing/postalcode', 'required': False}),
            ('billing_country', {'translated_name': 'billing/country', 'required': False}),
            ('billing_phone', {'translated_name': 'billing/phone', 'required': False}),
            ('billing_email', {'translated_name': 'billing/email', 'required': False}),
            ('billing_company_name', {'translated_name': 'billing/companyName', 'required': False}),

            ('shipping_name', {'translated_name': 'shipping/name', 'required': False}),
            ('shipping_address', {'translated_name': 'shipping/address', 'required': False}),
            ('shipping_address2', {'translated_name': 'shipping/address2', 'required': False}),
            ('shipping_city', {'translated_name': 'shipping/city', 'required': False}),
            ('shipping_state', {'translated_name': 'shipping/state', 'required': False}),
            ('shipping_postalcode', {'translated_name': 'shipping/postalcode', 'required': False}),
            ('shipping_country', {'translated_name': 'shipping/country', 'required': False}),
            ('shipping_phone', {'translated_name': 'shipping/phone', 'required': False}),
            ('shipping_email', {'translated_name': 'shipping/email', 'required': False}),

            ('expiration_date', {'translated_name': 'transactionDetail/payType/boleto/expirationDate'}),
            ('number', {'translated_name': 'transactionDetail/payType/boleto/number'}),
            ('instructions', {'translated_name': 'transactionDetail/payType/boleto/instructions', 'required': False}),
            ('format', {'translated_name': 'transactionDetail/payType/boleto/format', 'required': False}),            
            ## desconto
            ('discount_date', {'translated_name': 'transactionDetail/payType/boleto/discount/date', 'required': False}),
            ('discount_type', {'translated_name': 'transactionDetail/payType/boleto/discount/type', 'required': False}),
            ('discount_value', {'translated_name': 'transactionDetail/payType/boleto/discount/value', 'required': False}),
            ## multa
            ('charge_date', {'translated_name': 'transactionDetail/payType/boleto/charge/date', 'required': False}),
            ('charge_type', {'translated_name': 'transactionDetail/payType/boleto/charge/type', 'required': False}),
            ('charge_value', {'translated_name': 'transactionDetail/payType/boleto/charge/value', 'required': False}),
            ## juros, taxas
            ('rate_date', {'translated_name': 'transactionDetail/payType/boleto/rate/date', 'required': False}),
            ('rate_type', {'translated_name': 'transactionDetail/payType/boleto/rate/type', 'required': False}),
            ('rate_value', {'translated_name': 'transactionDetail/payType/boleto/rate/value', 'required': False}),
            
            ('charge_total', {'translated_name': 'payment/chargeTotal'})
        )

        requester = PaymentRequester(fields, kwargs)
        return self.send(command='sale', requester=requester, resource=PaymentResource)

    def get_fraud_check_iframe(self, **kwargs):
        fields = (
            ('order_id', {}),
        )
        requester = PaymentRequester(fields, kwargs)

        params = {
            'm': self.maxid,
            's': requester.cleaned_data.get('order_id'),
            'h': md5('{0}*{1}'.format(self.maxid, requester.cleaned_data.get('order_id'))).hexdigest()
        }

        url = 'https://testauthentication.maxipago.net/redirection_service/logo?{0}'.format(urlencode(params))
        return '<iframe width="1" height="1" frameborder="0" src="{0}"></iframe>'.format(url)


TEST, REDECARD, AMEX, CIELO, TEF, CHASEPAYMENTECH, BSITAU, BSBRADESCO= '1', '2', '3', '4', '5', '8', '11', '12'

PROCESSORS_CHOICES = (
    (TEST, 'Simulador de testes'),
    (REDECARD, 'Redecard'),
    (AMEX, 'Amex'),
    (CIELO, 'Cielo'),
    (TEF, 'TEF'),
    (CHASEPAYMENTECH, 'ChasePaymentech'),
    (BSITAU, 'BSItau'),
    (BSBRADESCO, 'BSBradesco'),
)
